<?php
//
// phpIpacstats - A web frontend for ipac[-ng]
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// $Id: index.php,v 1.19 2002/04/19 18:14:22 bene1 Exp $
// file: index.php
// function: main script


//retrieve settings from config-file
require('config.php');

//set program version that is displayed in footer
$prog_version='0.4.1';

// default to english, but this is negotiable.
if (!(isset($lng) && file_exists('includes/lang/' . $lng . '.php'))) {
    $lng = $default_lang;
}

require('includes/lang/' . $lng . '.php');  // get our language include
require('includes/class.Template.inc.php');  // template library
require('includes/system_functions.php');    // finish defining all of our global functions and variables
require('includes/parse_output.php');

// figure out if we got a template passed in the url
if (!(isset($template) && file_exists("templates/$template"))) {
    // default template we should use if we don't get a argument.
    define('TEMPLATE_SET', $default_template);
    $template = $default_template;
} else {
    define('TEMPLATE_SET', $template);
}

// figure out if we got times or dates passed
if (!(isset($Start_Date))) {
    $Start_Date = 'YYYY/MM/DD';
}
if (!isset($Start_Time)) {
    $Start_Time = 'hh:mm:ss';
}
if (!isset($End_Date)) {
    $End_Date = date("Y/m/d");
}
if (!isset($End_Time)) {
    $End_Time = date("H:i:s");
}
if (!isset($Time_Method)) {
    $Time_Method = "Pre_Frame";
}

//split up date and time-strings into arrays
if ($Time_Method == "Abs") {
   $startdate_ar = explode("/",$Start_Date);
   $starttime_ar = explode(":",$Start_Time);
   $enddate_ar = explode("/",$End_Date);
   $endtime_ar = explode(":",$End_Time);
   $errorcode = 0;

    // do some input data checks

    require('includes/inputcheck.php'); 
    
}


// we need ipac.conf to proceed, so see if we can read it
if (!is_readable($ipac_conffile)){
    $errorcode = $errorcode | 16;
}

// see if ipacsum path is set correctly and ipacsum is executable
if (!is_executable($ipacsum_executable)) {
    $errorcode = $errorcode | 32;
}

$radiobuttons = array();

if ($errorcode >0) {
    $text['title'] .= ": ".$text['error']."!";
    switch ($Time_Method) {
	case('Abs'):  
	            $radiobuttons['Abs'] = 'checked';
		    break;
	default:
	case('Pre_Frame'):
		    $radiobuttons['Pre_Frame'] = 'checked';
		    break;
    }
} else {
    switch($Time_Method) {
	case ('Abs') :
			$radiobuttons['Abs'] = 'checked';
			
			// add leading '0' to dates/times if neccessary
			
			reset ($startdate_ar);
			next($startdate_ar);
			array_walk($startdate_ar, 'addzero');
			reset($starttime_ar);
			array_walk($starttime_ar, 'addzero');
			reset($enddate_ar);
			next($enddate_ar);
			array_walk($enddate_ar, 'addzero');
			reset($endtime_ar);
			array_walk($endtime_ar, 'addzero');
			
			$timestring = "-s $startdate_ar[0]$startdate_ar[1]$startdate_ar[2]"
				    . "$starttime_ar[0]$starttime_ar[1]$starttime_ar[2]"
				    . "  -e $enddate_ar[0]$enddate_ar[1]$enddate_ar[2]"
				    . "$endtime_ar[0]$endtime_ar[1]$endtime_ar[2]";
			break;
	case ('Pre_Frame'):
	default:	$radiobuttons['Pre_Frame'] = 'checked';
			if (!isset($timeframe)) {
			    $timeframe = 'today';
			}
			$text['title'] .= ": ".$text[$timeframe];
			$timestring = "-t \"$timeframe\"";
    }
}

// adds leading '0' if neccessary
function addzero(&$value, $key) {
    if (ereg("^[0-9]$",$value)) {
	$value = "0".$value;
    }
}


// fire up the template engine
$tpl = new Template(dirname(__FILE__) . '/templates/' . TEMPLATE_SET);
$tpl->set_file(array(
    'form' => 'form.tpl'
));


// print out a box of information
function makebox ($title, $content)
{
    $t = new Template(dirname(__FILE__) . '/templates/' . TEMPLATE_SET);

    $t->set_file(array(
        'box'  => 'box.tpl'
    ));

    $t->set_var('title', $title);
    $t->set_var('content', $content);

    return $t->fp('out', 'box');
}  

//add passed string to global string errormsg with some formatting
function add_error($errortype) {
    global $errormsg,$text;
    $errormsg .= <<<EOD
    <li class="error">{$text[$errortype]}</li>
EOD;
}


// if error occured, print error page and stop
if ($errorcode != 0) {
    $errormsg = <<<EOD
<p class="error">{$text[an_error_occured]}
<ul>
EOD;

// figure out which input fields were wrong
    if (($errorcode & 16) == 16)
	add_error("ipac_conf_not_readable");
    if (($errorcode & 32) == 32)
	add_error("ipacsum_not_exec");
    if (($errorcode & 1) == 1) 
	add_error("startdate_is_wrong");
    if (($errorcode & 2) == 2) 
	add_error("starttime_is_wrong");
    if (($errorcode & 4) == 4) 
	add_error("enddate_is_wrong");
    if (($errorcode & 8) == 8) 
	add_error("endtime_is_wrong");

    $errormsg .= "</ul>";

    // let the page begin...

    require('includes/system_header.php');
    $tpl->set_var('title', $text['title']);

    $tpl->set_var('textstats', makebox($text['error'], $errormsg));

    //ok let's show the images...
    //$tpl->set_var('input', makebox($text['input'], ""));

    //$tpl->set_var('output', makebox($text['output'], ""));

    require('includes/configuration.php');

    require('includes/about.php');

    // parse our the template
    $tpl->pfp('out', 'form');

    print "</body></html>";
    exit();

}

//okay, no errors, now hurry on...

//get rulenames and seperate in/out-coming and "both" rules

//print "<pre>";
$in_rules = array();	    //rule-names of incoming rules saved here
$out_rules = array();	    //rule-names of outgoing rules saved here
$in_files = array();	    //filenames of the images of incoming rules
$out_files = array();	    // -- same for outgoing

if (is_readable($ipac_conffile)){
    if ($fp = fopen($ipac_conffile, 'r')) {
	while (!feof($fp)) {
	    $row = fgets($fp, 4096);
	    if ($row[0]!='#') {
		unset($fields);
		ereg("([a-zA-Z0-9_\. ]+)\|([a-zA-Z ]+)\|",$row,$fields);
		if ($fields[2]=='in') 
		    array_push($in_rules,$fields[1]);
		elseif ($fields[2]=='out') 
		    array_push($out_rules,$fields[1]);
	    } 
	}
	fclose($fp);
    } else { print "error opening conffile"; }
} else { print "error reading conffile"; }


//remove duplicates (are being added by ipacsum)
$in_rules = array_values(array_unique($in_rules));
$out_rules = array_values(array_unique($out_rules));

//create filenames out of rulenames by substituting space,\,/,|,<,> by _
for ($i=0;$i<count($in_rules);$i++) {
    $in_files[$i]  = preg_replace("/[\s\\/\|\>\<]/","_",$in_rules[$i]);
}
for ($i=0;$i<count($out_rules);$i++) {
    $out_files[$i] = preg_replace("/[\s\\/\|\>\<]/","_",$out_rules[$i]);
}

/*
$new = array();

for($i=0;$i<count($config_in_ar);$i++){
    if(in_array($config_in_ar[$i]["rulename"], $new) != "true"){
	$new[] = $config_in_ar[$i];
    }
}
*/

// just to test parsing...
/*
for ($i=0;$i<count($in_rules);$i++){
  print $in_rules[$i] ." -&gt ";
  print $in_files[$i];
  print "<br>";
}
for ($i=0;$i<count($out_rules);$i++){
  print $out_rules[$i] ." -&gt ";
  print $out_files[$i];
  print "<br>";
}
print "</pre>";
*/


//define parameters for the ipacsum-program
$ipac_args=$timestring." --png $data_dir --png-average-curve 15 ";
$ipac_args.=' --png-height '.$img_height.' --png-width '.$img_width;

//print $ipac_args;

//call the ipacsum-program

if ($fp = popen("$ipacsum_executable $ipac_args", 'r')) {
       while (!feof($fp)) {
          $execreturn .= fgets($fp, 4096);
       }
       $execreturn=trim($execreturn);
}

// only show images if ipacsum created some output
if ($execreturn == "") {
    $textstats = <<<EOD
<br><br>
<div align="center">
<b>{$text['no_output']}</b>
</div>
<br><br>
EOD;
} else {
    $textstats = parse_output($execreturn, $in_rules, $out_rules);
    
    // generate html-code for images on incoming-side
    $in_image_code = create_code($in_rules, $in_files);

    //generate html-code for images on outgoing-side
    $out_image_code = create_code($out_rules, $out_files);
}

//return html-code where all images are displayed
function create_code($rules, $files) {
    global $text,$img_width,$img_height,$img_mode;
    unset($code);
    if (is_readable("data/$files[0].png")) {
	$extension = "png";
    } else {
	$extension = "gif";
    }
    for ($i=0;$i<count($rules);$i++) {
        $code .= "	\n	<p>{$text[rule]}: $rules[$i]<br>	<div align=\"center\">";
	switch ($img_mode) {
	    case 'php': 
		$code .= '<img src="image.php?image=data/'.$files[$i].'.png" height="'.$img_height.'" ' 
		         .'width="'.$img_width.'" alt="Statistics of rule '.$rules[$i].'">';
	    break;
	    case 'disk':
		$code .= '<img src="data/'.$files[$i].'.'.$extension.'" height="'.$img_height.'" ' 
			.'width="'.$img_width.'" alt="Statistics of rule '.$rules[$i].'">';
	    break;
	}
	$code .= '</div>';
    }
    return $code;
}



// ************************************************************************ 
// let the page begin.

require('includes/system_header.php');
$tpl->set_var('title', $text['title']);

//print "<br>".$ipacsum_executable." ".$ipac_args."<br>";
//print $Time_Method;

$tpl->set_var('textstats', makebox($text['textstats'], $textstats));

//ok let's show the images...
if ($in_image_code != "") {
    $tpl->set_var('input', makebox($text['input'], $in_image_code));
}
if ($out_image_code != "") {
    $tpl->set_var('output', makebox($text['output'], $out_image_code));
}

require('includes/configuration.php');

require('includes/about.php');

// parse our the template
$tpl->pfp('out', 'form');

?>
</body>
</html>
